<?php
/**
 * Drip Content Schedule Database class
 *
 * @since 1.0
 */
namespace RCP\Addon\DripContent\Schedules;

defined( 'WPINC' ) or die;

final class Drip_Schedule_DB implements \RCP\Addon\DripContent\Interfaces\Drip_Schedule_DB {

	/**
	 * The drip schedule's properties.
	 *
	 * @access private
	 * @var array
	 */
	private $schedule;

	/**
	 * Holds the current WordPress core $wpdb global. Used internally for convenience.
	 *
	 * @access private
	 * @var wpdb object
	 */
	private $wpdb;

	/**
	 * The drip schedule table name.
	 *
	 * @access private
	 * @var string
	 */
	private $table;

	/**
	 *
	 * @param array $schedule {
	 * 	An array containing the schedule properties.
	 *
	 * 	@type int    $id            The schedule ID.
	 * 	@type int    $level_id      The subscription level ID.
	 * 	@type string $type          The drip schedule type. 'post' or 'taxonomy'.
	 * 	@type string $taxonmy       The taxonomy name for taxonomy schedule types.
	 * 	@type int    $taxonomy_term The taxonomy term ID for taxonomy schedule types.
	 * 	@type int    $post_id       The post ID for the 'post' schedule type.
	 * 	@type int    $unit          The duration unit. Example: 1
	 * 	@type string $period        The duration period. 'day', 'month', or 'year'.
	 * }
	 */
	public function __construct( $schedule ) {

		$this->schedule = wp_parse_args( $schedule, array(
			'id'            => '',
			'level_id'      => '',
			'type'          => 'post',
			'taxonomy'      => '',
			'taxonomy_term' => '',
			'post_id'       => '',
			'unit'          => 1,
			'period'        => 'day'
		) );

		global $wpdb;

		$this->wpdb = $wpdb;

		$this->table = $wpdb->prefix . 'rcp_drip_schedules';
	}

	/**
	 * Creates the drip schedule using the schedule data the class was instantiated with.
	 */
	public function create_schedule() {

		if ( ! $this->valid_type() ) {
			return;
		}

		return $this->wpdb->query( $this->wpdb->prepare(
			"INSERT INTO {$this->table} SET
				`level_id`      = %d,
				`type`          = %s,
				`taxonomy`      = %s,
				`taxonomy_term` = %d,
				`post_id`       = %d,
				`unit`          = %d,
				`period`        = %s
			;",
			absint( $this->schedule['level_id'] ),
			sanitize_text_field( $this->schedule['type'] ),
			sanitize_text_field( $this->schedule['taxonomy'] ),
			absint( $this->schedule['taxonomy_term'] ),
			absint( $this->schedule['post_id'] ),
			absint( $this->schedule['unit'] ),
			sanitize_text_field( $this->schedule['period'] )
		) );
	}

	/**
	 * Retrieves all the schedules for the specified level.
	 */
	public function schedules_for_level() {

		if ( empty( $this->schedule['level_id'] ) ) {
			return;
		}

		return $this->wpdb->get_results( $this->wpdb->prepare( "SELECT * FROM {$this->table} WHERE level_id = %d", $this->schedule['level_id'] ), ARRAY_A );
	}

	/**
	 * Retrieves all the schedules for the specified post.
	 */
	public function schedules_for_post() {

		if ( empty( $this->schedule['post_id'] ) ) {
			return;
		}

		return $this->wpdb->get_results( $this->wpdb->prepare( "SELECT * FROM {$this->table} WHERE post_id = %d", $this->schedule['post_id'] ), ARRAY_A );
	}

	/**
	 * Retrieves the specified drip schedule by its ID.
	 */
	public function schedule_by_id() {

		if ( empty( $this->schedule['id'] ) ) {
			return;
		}

		return $this->wpdb->get_results( $this->wpdb->prepare( "SELECT * FROM {$this->table} WHERE id = %d", $this->schedule['id'] ), ARRAY_A );
	}

	/**
	 * Updates the specified drip schedule.
	 */
	public function update_schedule() {

		if ( ! $this->valid_type() ) {
			return;
		}

		if ( empty( $this->schedule['id'] ) ) {
			return;
		}

		return $this->wpdb->query( $this->wpdb->prepare(
			"UPDATE {$this->table} SET
				`level_id`      = %d,
				`type`          = %s,
				`taxonomy`      = %s,
				`taxonomy_term` = %d,
				`post_id`       = %d,
				`unit`          = %d,
				`period`        = %s
			WHERE `id` = %d;",
			absint( $this->schedule['level_id'] ),
			sanitize_text_field( $this->schedule['type'] ),
			sanitize_text_field( $this->schedule['taxonomy'] ),
			absint( $this->schedule['taxonomy_term'] ),
			absint( $this->schedule['post_id'] ),
			absint( $this->schedule['unit'] ),
			sanitize_text_field( $this->schedule['period'] ),
			absint( $this->schedule['id'] )
		) );
	}

	/**
	 * Deletes the specified drip schedule.
	 */
	public function delete_schedule() {

		if ( empty( $this->schedule['id'] ) ) {
			return;
		}

		return $this->wpdb->query( $this->wpdb->prepare( "DELETE FROM {$this->table} WHERE id = %d", absint( $this->schedule['id'] ) ) );
	}

	/**
	 * Deletes all the drip schedules for the specified post.
	 */
	public function delete_all_schedules_for_post() {

		if ( empty( $this->schedule['post_id'] ) ) {
			return;
		}

		return $this->wpdb->query( $this->wpdb->prepare( "DELETE FROM {$this->table} WHERE post_id = %d", absint( $this->schedule['post_id'] ) ) );
	}

	/**
	 * Deletes all the drip schedules for the specified taxonomy term.
	 */
	public function delete_all_schedules_for_taxonomy_term() {

		if ( empty( $this->schedule['taxonomy_term'] ) ) {
			return;
		}

		return $this->wpdb->query( $this->wpdb->prepare( "DELETE FROM {$this->table} WHERE taxonomy_term = %d", absint( $this->schedule['taxonomy_term'] ) ) );
	}

	/**
	 * Deletes all the drip schedules for the specified subscription level.
	 */
	public function delete_all_schedules_for_level() {

		if ( empty( $this->schedule['level_id'] ) ) {
			return;
		}

		return $this->wpdb->query( $this->wpdb->prepare( "DELETE FROM {$this->table} WHERE level_id = %d", absint( $this->schedule['level_id'] ) ) );
	}

	/**
	 * Checks if the instantiated schedule is valid.
	 *
	 * @access public
	 * @since 1.0
	 * @return bool True if the schedule is valid, false if it is not.
	 */
	public function valid_type() {

		$valid = true;

		if ( ! isset( $this->schedule['unit'] ) || empty( $this->schedule['period'] ) ) {
			$valid = false;
		}

		if ( 'post' === $this->schedule['type'] && empty( $this->schedule['post_id'] ) ) {
			$valid = false;
		}

		if ( 'taxonomy' === $this->schedule['type'] && ( empty( $this->schedule['taxonomy'] ) || empty( $this->schedule['taxonomy_term'] ) ) ) {
			$valid = false;
		}

		if ( ! in_array( $this->schedule['period'], array( 'day', 'month', 'year' ) ) ) {
			$valid = false;
		}

		return $valid;
	}
}