<?php
/**
 * Register settings
 *
 * @package     RCP\IP_Restriction\Admin\Settings\Register
 * @since       1.0.0
 */


// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}


/**
 * Register the IP Restriction settings
 *
 * @since       1.0.0
 * @return      void
 */
function rcp_ip_restriction_register_settings() {
	register_setting( 'rcp_ip_restriction_settings_group', 'rcp_ip_restriction_settings', 'rcp_ip_restriction_sanitize_settings' );
}
add_action( 'admin_init', 'rcp_ip_restriction_register_settings', 100 );


/**
 * Sanitize settings
 *
 * @since       1.0.0
 * @param       array $settings The settings to save
 * @return      array $settings The settings to save
 */
function rcp_ip_restriction_sanitize_settings( $settings ) {
	$saved_settings = get_option( 'rcp_ip_restriction_settings' );

	if ( empty( $settings['license_key'] ) ) {
		delete_option( 'rcp_ip_restriction_license_status' );
	}

	if ( ! empty( $settings['whitelist'] ) ) {
		// Sanitize the input
		$whitelist = rcp_ip_restriction_sanitize_ips( $settings['whitelist'] );
	} else {
		$whitelist = array();
	}

	$settings['whitelist'] = $whitelist;

	if ( ! empty( $settings['blacklist'] ) ) {
		// Sanitize the input
		$blacklist = rcp_ip_restriction_sanitize_ips( $settings['blacklist'] );
	} else {
		$blacklist = array();
	}

	$settings['blacklist'] = $blacklist;

	return $settings;
}


/**
 * Add the IP Restriction menu item
 *
 * @since       1.0.0
 * @return      void
 */
function rcp_ip_restriction_admin_menu() {
	add_submenu_page(
		'rcp-members',
		__( 'IP Restriction Settings', 'rcp-ip-restriction' ),
		__( 'IP Restriction', 'rcp-ip-restriction' ),
		'manage_options',
		'rcp-ip-restriction',
		'rcp_ip_restriction_render_settings_page'
	);
}
add_action( 'admin_menu', 'rcp_ip_restriction_admin_menu', 100 );


/**
 * Render the settings page
 *
 * @since       1.0.0
 * @return      void
 */
function rcp_ip_restriction_render_settings_page() {
	$settings = get_option( 'rcp_ip_restriction_settings' );

	echo '<div class="wrap">';
	echo '<h2>' . esc_html( get_admin_page_title() ) . '</h2>';

	if ( isset( $_REQUEST['updated'] ) && $_REQUEST['updated'] !== false ) {
		echo '<div class="updated fade"><p><strong>' . __( 'Options saved', 'rcp-ip-restriction' ) . '</strong></p></div>';
	}

	if ( isset( $settings['whitelist'] ) && is_array( $settings['whitelist'] ) ) {
		$whitelist = implode( "\n", $settings['whitelist'] );
	} else {
		$whitelist = '';
	}

	if ( isset( $settings['blacklist'] ) && is_array( $settings['blacklist'] ) ) {
		$blacklist = implode( "\n", $settings['blacklist'] );
	} else {
		$blacklist = '';
	}

	if ( isset( $settings['blacklist_error'] ) ) {
		$blacklist_error = $settings['blacklist_error'];
	} else {
		$blacklist_error = __( 'An unknown error has occurred, please try again.', 'rcp-ip-restriction' );
	}
	?>

	<div id="rcp_ip_restriction_settings">
		<form method="post" action="options.php" class="rcp_options_form">
			<?php settings_fields( 'rcp_ip_restriction_settings_group' ); ?>
			<table class="form-table">
				<tr style="display: none">
					<th>
						<label for="rcp_ip_restriction_settings[locking]"><?php _e( 'IP Locking', 'rcp-aweber-pro' ); ?></label>
					</th>
					<td>
						<input type="checkbox" id="rcp_ip_restriction_settings[locking]" name="rcp_ip_restriction_settings[locking]" value="1" <?php echo( isset( $settings['locking'] ) ? checked( $settings['locking'], 1, false ) : '' ); ?>/>
						<span class="description"><?php _e( 'Check to enable locking subscriptions by IP address.', 'rcp-ip-restriction' ); ?></span>
					</td>
				</tr>
				<tr>
					<th>
						<label for="rcp_ip_restriction_settings[whitelist]"><?php _e( 'IP Whitelist', 'rcp-ip-restriction' ); ?></label>
					</th>
					<td>
						<textarea id="rcp_ip_restriction_settings[whitelist]" name="rcp_ip_restriction_settings[whitelist]" style="width: 500px; height: 200px;"><?php echo $whitelist; ?></textarea>
						<p class="description"><?php _e( 'Enter IP addresses to whitelist, one per line.', 'rcp-ip-restriction' ); ?></p>
					</td>
				</tr>
				<tr>
					<th>
						<label for="rcp_ip_restriction_settings[blacklist]"><?php _e( 'IP Blacklist', 'rcp-ip-restriction' ); ?></label>
					</th>
					<td>
						<textarea id="rcp_ip_restriction_settings[blacklist]" name="rcp_ip_restriction_settings[blacklist]" style="width: 500px; height: 200px;"><?php echo $blacklist; ?></textarea>
						<p class="description"><?php _e( 'Enter IP addresses to blacklist, one per line.', 'rcp-ip-restriction' ); ?></p>
					</td>
				</tr>
				<tr>
					<th>
						<label for="rcp_ip_restriction_settings[blacklist_error]"><?php _e( 'Blacklist Error Message', 'rcp-ip-restriction' ); ?></label>
					</th>
					<td>
						<input type="text" id="rcp_ip_restriction_settings[blacklist_error]" name="rcp_ip_restriction_settings[blacklist_error]" class="large-text" value="<?php echo $blacklist_error; ?>" />
						<p class="description"><?php _e( 'Customize the message displayed when a blacklisted address tries to subscribe.', 'rcp-ip-restriction' ); ?></p>
					</td>
				</tr>
				<?php if ( class_exists( 'S214_Debug' ) ) { ?>
					<?php if( isset( $settings['enable_debug'] ) ) { $enable_debug = $settings['enable_debug']; } ?>
					<tr>
						<th>
							<label for="rcp_ip_restriction_settings[enable_debug]"><?php _e( 'Enable Debugging', 'rcp-ip-restriction' ); ?></label>
						</th>
						<td>
							<input type="checkbox" id="rcp_ip_restriction_settings[enable_debug]" name="rcp_ip_restriction_settings[enable_debug]" value="1" <?php echo ( isset( $enable_debug ) ? checked( 1, $enable_debug, false ) : '' ); ?>/>
							<span class="description"><?php printf( __( 'Log plugin errors. You can view errors %s.', 'rcp-ip-restriction' ), '<a href="' . admin_url( 'tools.php?page=s214-debug-logs' ) . '">' . __( 'here', 'rcp-ip-restriction' ) . '</a>' ); ?></span>
						</td>
					</tr>
				<?php } ?>
			</table>

			<p class="submit">
				<input type="submit" class="button-primary" value="<?php _e( 'Save Options', 'rcp-ip-restriction' ); ?>" />
			</p>
		</form>
	</div>
	<div id="rcp_ip_restriction_instructions">
		<h4><?php _e( 'Instructions', 'rcp-ip-restriction' ); ?></h4>
		<p><?php printf( __( 'IP Restriction currently supports two modes, %s and %s.', 'rcp-ip-restriction' ), '<em>' . __( 'IP Whitelisting', 'rcp-ip-restriction' ) . '</em>', '<em>' . __( 'IP Blacklisting', 'rcp-ip-restriction' ) . '</em>' ); ?></p>
		<hr />
		<strong style="display: none"><?php _e( 'IP Locking', 'rcp-ip-restriction' ); ?></strong>
		<p style="display: none"><?php _e( 'This mode \'locks\' subscriptions to the IP address they are purchased from. If a user changes their IP address or connects from a different maching, they will have to re-subscribe under the new address.', 'rcp-ip-restriction' ); ?></p>
		<strong><?php _e( 'IP Whitelisting', 'rcp-ip-restriction' ); ?></strong>
		<p><?php _e( 'This mode allows you to specify IP addresses which can bypass the normal subscription requirements on your site. Addresses may be whitelisted globally, per subscription level or per post. Addresses must be entered one per line.', 'rcp-ip-restriction' ); ?></p>
		<p><?php printf( __( 'Your IP address: %s', 'rcp-ip-restriction' ), '<code>' . rcp_get_ip() . '</code>' ); ?></p>
		<strong><?php _e( 'IP Blacklisting', 'rcp-ip-restriction' ); ?></strong>
		<p><?php _e( 'This mode allows you to specify IP addresses which will not be able to purchase subscriptions. Note that this does not apply retroactively, it simply displays an error when a blacklisted address tries to subscribe. Addresses must be entered one per line.', 'rcp-ip-restriction' ); ?></p>
		<hr />
		<h4><?php _e( 'Formatting', 'rcp-ip-restriction' ); ?></h4>
		<p><?php _e( 'IP Restriction supports both IPv4 and IPv6 addresses, along with wildcards. If using wildcards, all octets are not required. For example, if you want to restrict everything in the <code>127.0</code> block, you only need to enter <code>127.0.*</code>, the fourth octet is not required.', 'rcp-ip-restriction' ); ?></p>

	</div>
	<?php
	echo '</div>';
}


/**
 * Add per-level setting fields
 *
 * @since       1.0.0
 * @return      void
 */
function rcp_ip_restriction_add_subscription_settings() {
	if ( isset( $_GET['edit_subscription'] ) ) {
		$whitelists = get_option( 'rcp_ip_restriction_subscription_whitelists' );

		if ( is_array( $whitelists ) && array_key_exists( $_GET['edit_subscription'], $whitelists ) && is_array( $whitelists[$_GET['edit_subscription']] ) ) {
			$whitelist = $whitelists[ $_GET['edit_subscription'] ];
		} else {
			$whitelist = array();
		}
	} else {
		$whitelist = array();
	}
	?>
		<tr class="form-field">
			<th scope="row" valign="top">
				<label for="rcp-ip-restriction-whitelist"><?php _e( 'IP Whitelist', 'rcp-ip-restriction' ); ?></label>
			</th>
			<td>
				<textarea id="rcp-ip-restriction-whitelist" name="rcp-ip-restriction-whitelist" style="width: 500px; height: 200px;"><?php echo implode( "\n", $whitelist ); ?></textarea>
				<p class="description"><?php _e( 'Enter IP addresses to whitelist, one per line.', 'rcp-ip-restriction' ); ?></p>
			</td>
		</tr>
	<?php
}
add_action( 'rcp_add_subscription_form', 'rcp_ip_restriction_add_subscription_settings' );
add_action( 'rcp_edit_subscription_form', 'rcp_ip_restriction_add_subscription_settings' );


/**
 * Store the whitelist in subscription meta
 *
 * @since       1.0.0
 * @param       int $level_id The subscription ID
 * @param       array $args Arguements passed to the action
 */
function rcp_ip_restriction_save_subscription( $level_id = 0, $args ) {
	$whitelists = get_option( 'rcp_ip_restriction_subscription_whitelists', array() );

	if ( ! empty( $_POST['rcp-ip-restriction-whitelist'] ) ) {
		$whitelists[ $level_id ] = rcp_ip_restriction_sanitize_ips( $_POST['rcp-ip-restriction-whitelist'] );

		update_option( 'rcp_ip_restriction_subscription_whitelists', $whitelists );
	} else {
		if ( array_key_exists( $level_id, $whitelists ) ) {
			unset( $whitelists[ $level_id ] );
		}

		if ( empty( $whitelists ) ) {
			delete_option( 'rcp_ip_restriction_subscription_whitelists' );
		} else {
			update_option( 'rcp_ip_restriction_subscription_whitelists', $whitelists );
		}
	}
}
add_action( 'rcp_add_subscription', 'rcp_ip_restriction_save_subscription', 10, 2 );
add_action( 'rcp_edit_subscription_level', 'rcp_ip_restriction_save_subscription', 10, 2 );
