<?php
/**
 * Register settings
 *
 * @package     RCP\RCP_Site_Creation\Admin\Settings\Register
 * @since       1.0.0
 */


// Exit if accessed directly
if( ! defined( 'ABSPATH' ) ) {
	exit;
}


/**
 * Register the Site_Creation settings
 *
 * @since       1.0.0
 * @return      void
 */
function rcp_site_creation_register_settings() {
	register_setting( 'rcp_site_creation_settings_group', 'rcp_site_creation_settings', 'rcp_site_creation_sanitize_settings' );
}
add_action( 'admin_init', 'rcp_site_creation_register_settings', 100 );


/**
 * Sanitize settings
 *
 * @since       1.0.0
 * @param       array $settings The settings to save
 * @return      array $settings The settings to save
 */
function rcp_site_creation_sanitize_settings( $settings ) {
	$saved_settings = get_option( 'rcp_site_creation_settings' );

	return $settings;
}


/**
 * Add the Site_Creation menu item
 *
 * @since       1.0.0
 * @return      void
 */
function rcp_site_creation_admin_menu() {
	add_submenu_page(
		'rcp-members',
		__( 'Site Creation Settings', 'rcp-site-creation' ),
		__( 'Site Creation', 'rcp-site-creation' ),
		'manage_options',
		'rcp-site-creation',
		'rcp_site_creation_render_settings_page'
	);
}
add_action( 'admin_menu', 'rcp_site_creation_admin_menu', 100 );


/**
 * Render the settings page
 *
 * @since       1.0.0
 * @return      void
 */
function rcp_site_creation_render_settings_page() {
	$settings = get_option( 'rcp_site_creation_settings' );

	echo '<div class="wrap">';
	echo '<h2>' . esc_html( get_admin_page_title() ) . '</h2>';

	if( isset( $_REQUEST['updated'] ) && $_REQUEST['updated'] !== false ) {
		echo '<div class="updated fade"><p><strong>' . __( 'Options saved', 'rcp-site-creation' ) . '</strong></p></div>';
	}

	if( isset( $settings['redirect_page'] ) ) {
		$redirect_page = $settings['redirect_page'];
	} else {
		$redirect_page = 'inherit';
	}

	if( isset( $settings['safe_urls'] ) ) {
		$safe_urls = 'checked="checked"';
	} else {
		$safe_urls = '';
	}

	$schemes  = ( ! empty( $settings['domain_mapping_schemes'] ) && is_array( $settings['domain_mapping_schemes'] ) ) ? $settings['domain_mapping_schemes'] : array( 'http' );

	if( isset( $settings['default_theme'] ) ) {
		$default_theme = $settings['default_theme'];
	} else {
		$theme_object = WP_Theme::get_core_default_theme();
		$default_theme = is_object( $theme_object ) ? $theme_object->get_stylesheet() : '';
	}

	if( isset( $settings['url_field_title'] ) ) {
		$url_field_title = $settings['url_field_title'];
	} else {
		$url_field_title = __( 'Subsite URL', 'rcp-site-creation' );
	}

	if( isset( $settings['name_field_title'] ) ) {
		$name_field_title = $settings['name_field_title'];
	} else {
		$name_field_title = __( 'Subsite Name', 'rcp-site-creation' );
	}

	if( isset( $settings['desc_field_show'] ) ) {
		$desc_field_show = 'checked="checked"';
	} else {
		$desc_field_show = '';
	}

	if( isset( $settings['desc_field_title'] ) ) {
		$desc_field_title = $settings['desc_field_title'];
	} else {
		$desc_field_title = __( 'Subsite Description', 'rcp-site-creation' );
	}

	if( isset( $settings['subsite_required_error'] ) ) {
		$subsite_required_error = $settings['subsite_required_error'];
	} else {
		$subsite_required_error = __( 'The subsite URL field is required, please try again.', 'rcp-site-creation' );
	}

	if( isset( $settings['subsite_exists_error'] ) ) {
		$subsite_exists_error = $settings['subsite_exists_error'];
	} else {
		$subsite_exists_error = __( 'The specified subsite URL is not available, please try again.', 'rcp-site-creation' );
	}
	?>

	<div id="rcp_site_creation_settings">
		<form method="post" action="options.php" class="rcp_options_form">
			<?php settings_fields( 'rcp_site_creation_settings_group' ); ?>
			<?php $pages = get_pages(); ?>
			<table class="form-table">
				<tr>
					<th colspan="2">
						<h3><?php _e( 'General', 'rcp-site-creation' ); ?></h3>
					</th>
				</tr>
				<tr>
					<th>
						<label for="rcp_site_creation_settings[redirect_page]"><?php _e( 'Success Page', 'rcp' ); ?></label>
					</th>
					<td>
						<select id="rcp_site_creation_settings[redirect_page]" name="rcp_site_creation_settings[redirect_page]">
							<option value="inherit" <?php echo selected( 'inherit', $redirect_page, false ); ?>><?php _e( '- Use default success page -', 'rcp-site-creation' ); ?></option>
							<option value="subsite" <?php echo selected( 'subsite', $redirect_page, false ); ?>><?php _e( '- Use new subsite -', 'rcp-site-creation' ); ?></option>

							<?php
							if( $pages ) {
								foreach( $pages as $page ) {
								  	$option  = '<option value="' . $page->ID . '" ' . selected( $page->ID, $redirect_page, false ) . '>';
									$option .= $page->post_title;
									$option .= ' (ID: ' . $page->ID . ')';
									$option .= '</option>';
									echo $option;
								}
							} else {
								echo '<option>' . __( 'No pages found', 'rcp-site-creation' ) . '</option>';
							}
							?>
						</select>
						<p class="description"><?php _e( 'This is the page users are redirected to after a successful subsite registration.', 'rcp-site-creation' ); ?></p>
					</td>
				</tr>
				<?php if( ! is_subdomain_install() ) { ?>
					<tr>
						<th>
							<label for="rcp_site_creation_settings[safe_urls]"><?php _e( 'Safe URL Check', 'rcp-site-creation' ); ?></label>
						</th>
						<td>
							<input type="checkbox" name="rcp_site_creation_settings[safe_urls]" id="rcp_site_creation_settings[safe_urls]" <?php echo $safe_urls; ?> />
							<label for="rcp_site_creation_settings[safe_urls]"><span class="description"><?php _e( 'Check to enable safe URL validation. This will prevent the registration of any blog with the same name as a page on the main site.', 'rcp-site-creation' ); ?></span></label>
						</td>
					</tr>
				<?php } ?>
				<tr>
					<th>
						<label for="rcp_site_creation_settings[domain_mapping]"><?php _e( 'Domain Mapping', 'rcp-site-creation' ); ?></label>
					</th>
					<td>
						<input type="checkbox" name="rcp_site_creation_settings[domain_mapping]" id="rcp_site_creation_settings[domain_mapping]" <?php checked( isset( $settings['domain_mapping'] ) ); ?>>
						<label for="rcp_site_creation_settings[domain_mapping]"><span class="description"><?php _e( 'Check to enable domain mapping. This allows members to enter custom domain names for their sites. If unchecked, they will choose a subdomain or subdirectory instead.', 'rcp-site-creation' ); ?></span></label>
					</td>
				</tr>
				<tr<?php echo ! isset( $settings['domain_mapping'] ) ? ' style="display: none;"' : ''; ?>>
					<th>
						<label for="rcp_site_creation_settings[domain_mapping_schemes]"><?php _e( 'Domain Mapping Schemes', 'rcp-site-creation' ); ?></label>
					</th>
					<td>
						<input type="checkbox" id="rcp_site_creation_settings[domain_mapping_schemes][http]" name="rcp_site_creation_settings[domain_mapping_schemes][]" value="http" <?php checked( in_array( 'http', $schemes ) ); ?>>
						<label for="rcp_site_creation_settings[domain_mapping_schemes][http]"><?php _e( 'http://', 'rcp-site-creation' ); ?></label> <br>
						<input type="checkbox" id="rcp_site_creation_settings[domain_mapping_schemes][https]" name="rcp_site_creation_settings[domain_mapping_schemes][]" value="https" <?php checked( in_array( 'https', $schemes ) ); ?>>
						<label for="rcp_site_creation_settings[domain_mapping_schemes][https]"><?php _e( 'https://', 'rcp-site-creation' ); ?></label>
						<p class="description"><?php _e( 'Select which scheme(s) you\'d like to allow for mapped domains.', 'rcp-site-creation' ); ?></p>
					</td>
				</tr>
				<tr>
					<th>
						<label for="rcp_site_creation_settings[default_theme]"><?php _e( 'Default Theme', 'rcp-site-creation' ); ?></label>
					</th>
					<td>
						<select id="rcp_site_creation_settings[default_theme]" name="rcp_site_creation_settings[default_theme]">
							<?php foreach ( wp_get_themes() as $theme ) : ?>
								<option value="<?php echo esc_attr( $theme->get_stylesheet() ); ?>" <?php selected( $default_theme, $theme->get_stylesheet() ); ?>><?php echo esc_html( $theme->title ); ?></option>
							<?php endforeach; ?>
						</select>
					</td>
				</tr>
				<tr>
					<th colspan="2">
						<h3><?php _e( 'Fields', 'rcp-site-creation' ); ?></h3>
					</th>
				</tr>
				<tr>
					<th>
						<label for="rcp_site_creation_settings[url_field_title]"><?php _e( 'Subsite URL Field Title', 'rcp-site-creation' ); ?></label>
					</th>
					<td>
						<input type="text" id="rcp_site_creation_settings[url_field_title]" name="rcp_site_creation_settings[url_field_title]" class="regular-text" value="<?php echo $url_field_title; ?>" />
						<p class="description"><?php _e( 'Customize the field title for the URL field on the subscription form.', 'rcp-site-creation' ); ?></p>
					</td>
				</tr>
				<tr>
					<th>
						<label for="rcp_site_creation_settings[name_field_title]"><?php _e( 'Subsite Name Field Title', 'rcp-site-creation' ); ?></label>
					</th>
					<td>
						<input type="text" id="rcp_site_creation_settings[name_field_title]" name="rcp_site_creation_settings[name_field_title]" class="regular-text" value="<?php echo $name_field_title; ?>" />
						<p class="description"><?php _e( 'Customize the field title for the name field on the subscription form.', 'rcp-site-creation' ); ?></p>
					</td>
				</tr>
				<tr>
					<th>
						<label for="rcp_site_creation_settings[desc_field_show]"><?php _e( 'Show Description Field', 'rcp-site-creation' ); ?></label>
					</th>
					<td>
						<input type="checkbox" name="rcp_site_creation_settings[desc_field_show]" id="rcp_site_creation_settings[desc_field_show]" <?php echo $desc_field_show; ?> />
						<label for="rcp_site_creation_settings[desc_field_show]"><span class="description"><?php _e( 'Check to show the site description field on the registration form.', 'rcp-site-creation' ); ?></span></label>
					</td>
				</tr>
				<tr>
					<th>
						<label for="rcp_site_creation_settings[desc_field_title]"><?php _e( 'Subsite Description Field Title', 'rcp-site-creation' ); ?></label>
					</th>
					<td>
						<input type="text" id="rcp_site_creation_settings[desc_field_title]" name="rcp_site_creation_settings[desc_field_title]" class="regular-text" value="<?php echo $desc_field_title; ?>" />
						<p class="description"><?php _e( 'Customize the field title for the description field on the subscription form.', 'rcp-site-creation' ); ?></p>
					</td>
				</tr>
				<tr>
					<th colspan="2">
						<h3><?php _e( 'Errors', 'rcp-site-creation' ); ?></h3>
					</th>
				</tr>
				<tr>
					<th>
						<label for="rcp_site_creation_settings[subsite_required_error]"><?php _e( 'Missing URL Error', 'rcp-site-creation' ); ?></label>
					</th>
					<td>
						<input type="text" id="rcp_site_creation_settings[subsite_required_error]" name="rcp_site_creation_settings[subsite_required_error]" class="regular-text" value="<?php echo $subsite_required_error; ?>" />
						<p class="description"><?php _e( 'Customize the error shown when a subsite URL is not specified.', 'rcp-site-creation' ); ?></p>
					</td>
				</tr>
				<tr>
					<th>
						<label for="rcp_site_creation_settings[subsite_exists_error]"><?php _e( 'Unavailable URL Error', 'rcp-site-creation' ); ?></label>
					</th>
					<td>
						<input type="text" id="rcp_site_creation_settings[subsite_exists_error]" name="rcp_site_creation_settings[subsite_exists_error]" class="regular-text" value="<?php echo $subsite_exists_error; ?>" />
						<p class="description"><?php _e( 'Customize the error shown when a specified subsite URL is not available.', 'rcp-site-creation' ); ?></p>
					</td>
				</tr>
			</table>

			<p class="submit">
				<input type="submit" class="button-primary" value="<?php _e( 'Save Options', 'rcp-site-creation' ); ?>" />
			</p>
		</form>
	</div>
	<?php
	echo '</div>';
}


/**
 * Add per-level setting fields
 *
 * @since       1.0.0
 * @return      void
 */
function rcp_site_creation_add_subscription_settings() {
	if( isset( $_GET['edit_subscription'] ) ) {
		if( rcp_site_creation_allowed( $_GET['edit_subscription'] ) ) {
			$allow_site_creation = 'checked="checked"';
		} else {
			$allow_site_creation = '';
		}

		$upload_quota = rcp_site_creation_get_upload_quota( absint( $_GET['edit_subscription'] ) );
	} else {
		$allow_site_creation = '';
		$upload_quota = '';
	}
	?>
		<tr class="form-field">
			<th scope="row" valign="top">
				<label for="rcp-site-creation-allow"><?php _e( 'Allow Subsite Creation', 'rcp-site-creation' ); ?></label>
			</th>
			<td>
				<input type="checkbox" name="rcp-site-creation-allow" id="rcp-site-creation-allow" <?php echo $allow_site_creation; ?> />
				<label for="rcp-site-creation-allow"><span class="description"><?php _e( 'Check to enable site creation for this subscription level.', 'rcp-site-creation' ); ?></span></label>
			</td>
		</tr>

        <tr class="form-field">
            <th scope="row" valign="top">
                <label for="rcp-site-creation-upload-quota"><?php _e( 'Upload Quota', 'rcp-site-creation' ); ?></label>
            </th>
            <td>
                <input type="number" name="rcp-site-creation-upload-quota" id="rcp-site-creation-upload-quota" value="<?php echo esc_attr( $upload_quota ); ?>" step="1" min="0" style="width: 100px;" />
                <label for="rcp-site-creation-upload-quota"><span class="description"><?php _e( 'MB (Leave blank for network default)', 'rcp-site-creation' ); ?></span></label>
            </td>
        </tr>
	<?php
}
add_action( 'rcp_add_subscription_form', 'rcp_site_creation_add_subscription_settings' );
add_action( 'rcp_edit_subscription_form', 'rcp_site_creation_add_subscription_settings' );


/**
 * Store the whitelist in subscription meta
 *
 * @since       1.0.0
 * @param       int $level_id The subscription ID
 * @param       array $args Arguements passed to the action
 */
function rcp_site_creation_save_subscription( $level_id = 0, $args ) {
	$allow_site_creation = get_option( 'rcp_site_creation_subscription_allow_site_creation' );

	if( ! empty( $_POST['rcp-site-creation-allow'] ) ) {
		$allow_site_creation[$level_id] = 'true';
	} else {
		unset( $allow_site_creation[$level_id] );
	}

	if( count( $allow_site_creation ) == 0 ) {
		delete_option( 'rcp_site_creation_subscription_allow_site_creation' );
	} else {
		update_option( 'rcp_site_creation_subscription_allow_site_creation', $allow_site_creation );
	}

	// Save upload quota.
	if( ! empty( $_POST['rcp-site-creation-upload-quota'] ) ) {
		rcp_site_creation_set_upload_quota( $level_id, absint( $_POST['rcp-site-creation-upload-quota'] ) );
	} else {
		rcp_site_creation_remove_upload_quota( $level_id );
	}
}
add_action( 'rcp_add_subscription', 'rcp_site_creation_save_subscription', 10, 2 );
add_action( 'rcp_edit_subscription_level', 'rcp_site_creation_save_subscription', 10, 2 );
